@@ -1,7 +1,9 @@
 from __future__ import print_function
 
 import argparse
+import io
 import itertools
+import json
 import os
 import random
 import re
@@ -17,9 +19,11 @@ from threading import Thread
 
 import pluggy
 import py
+import six
 import toml
 from packaging import requirements
 from packaging.utils import canonicalize_name
+from packaging.version import Version
 
 import tox
 from tox.constants import INFO
@@ -57,9 +61,9 @@ SUICIDE_TIMEOUT = 0.0
 INTERRUPT_TIMEOUT = 0.3
 TERMINATE_TIMEOUT = 0.2
 
-_FACTOR_LINE_PATTERN = re.compile(r"^([\w{}\.!,-]+)\:\s+(.+)")
-_ENVSTR_SPLIT_PATTERN = re.compile(r"((?:\{[^}]+\})+)|,")
-_ENVSTR_EXPAND_PATTERN = re.compile(r"\{([^}]+)\}")
+_FACTOR_LINE_PATTERN = re.compile(r"^([\w{}.!,-]+):\s+(.+)")
+_ENVSTR_SPLIT_PATTERN = re.compile(r"((?:{[^}]+})+)|,")
+_ENVSTR_EXPAND_PATTERN = re.compile(r"{([^}]+)}")
 _WHITESPACE_PATTERN = re.compile(r"\s+")
 
 
@@ -240,8 +244,8 @@ class PosargsOption:
 
 class InstallcmdOption:
     name = "install_command"
-    type = "argv"
-    default = "python -m pip install {opts} {packages}"
+    type = "argv_install_command"
+    default = r"python -m pip install \{opts\} \{packages\}"
     help = "install command for dependencies and package under test."
 
     def postprocess(self, testenv_config, value):
@@ -300,7 +304,7 @@ def parseconfig(args, plugins=()):
 
 
 def get_py_project_toml(path):
-    with open(str(path)) as file_handler:
+    with io.open(str(path), encoding="UTF-8") as file_handler:
         config_data = toml.load(file_handler)
         return config_data
 
@@ -390,9 +394,7 @@ class SetenvDict(object):
                 return os.environ.get(name, default)
             self._lookupstack.append(name)
             try:
-                res = self.reader._replace(val)
-                res = res.replace("\\{", "{").replace("\\}", "}")
-                self.resolved[name] = res
+                self.resolved[name] = res = self.reader._replace(val, name="setenv")
             finally:
                 self._lookupstack.pop()
             return res
@@ -410,6 +412,19 @@ class SetenvDict(object):
         self.definitions[name] = value
         self.resolved[name] = value
 
+    def items(self):
+        return ((name, self[name]) for name in self.definitions)
+
+    def export(self):
+        # post-process items to avoid internal syntax/semantics
+        # such as {} being escaped using \{\}, suitable for use with
+        # os.environ .
+        return {
+            name: Replacer._unescape(value)
+            for name, value in self.items()
+            if value is not self._DUMMY
+        }
+
 
 @tox.hookimpl
 def tox_addoption(parser):
@@ -559,6 +574,16 @@ def tox_addoption(parser):
         action="store_true",
         help="override alwayscopy setting to True in all envs",
     )
+    parser.add_argument(
+        "--no-provision",
+        action="store",
+        nargs="?",
+        default=False,
+        const=True,
+        metavar="REQUIRES_JSON",
+        help="do not perform provision, but fail and if a path was provided "
+        "write provision metadata as JSON to it",
+    )
 
     cli_skip_missing_interpreter(parser)
     parser.add_argument("--workdir", metavar="PATH", help="tox working directory")
@@ -570,7 +595,10 @@ def tox_addoption(parser):
     )
 
     def _set_envdir_from_devenv(testenv_config, value):
-        if testenv_config.config.option.devenv is not None:
+        if (
+            testenv_config.config.option.devenv is not None
+            and testenv_config.envname != testenv_config.config.provision_tox_env
+        ):
             return py.path.local(testenv_config.config.option.devenv)
         else:
             return value
@@ -1000,7 +1028,7 @@ class TestenvConfig:
         #: set of factors
         self.factors = factors
         self._reader = reader
-        self._missing_subs = []
+        self._missing_subs = {}
         """Holds substitutions that could not be resolved.
 
         Pre 2.8.1 missing substitutions crashed with a ConfigError although this would not be a
@@ -1008,12 +1036,31 @@ class TestenvConfig:
         check later when the testenv is actually run and crash only then.
         """
 
+    # Python 3 only, as __getattribute__ is ignored for old-style types on Python 2
+    def __getattribute__(self, name):
+        rv = object.__getattribute__(self, name)
+        if isinstance(rv, Exception):
+            raise rv
+        return rv
+
+    if six.PY2:
+
+        def __getattr__(self, name):
+            if name in self._missing_subs:
+                raise self._missing_subs[name]
+            raise AttributeError(name)
+
     def get_envbindir(self):
         """Path to directory where scripts/binaries reside."""
         is_bin = (
             isinstance(self.python_info, NoInterpreterInfo)
             or tox.INFO.IS_WIN is False
             or self.python_info.implementation == "Jython"
+            or (
+                # this combination is MSYS2
+                tox.INFO.IS_WIN
+                and self.python_info.os_sep == "/"
+            )
             or (
                 tox.INFO.IS_WIN
                 and self.python_info.implementation == "PyPy"
@@ -1032,7 +1079,7 @@ class TestenvConfig:
         return self.get_envpython()
 
     def get_envpython(self):
-        """ path to python/jython executable. """
+        """path to python/jython executable."""
         if "jython" in str(self.basepython):
             name = "jython"
         else:
@@ -1268,11 +1315,11 @@ class ParseIni(object):
             feedback("--devenv requires only a single -e", sysexit=True)
 
     def handle_provision(self, config, reader):
-        requires_list = reader.getlist("requires")
+        config.requires = reader.getlist("requires")
         config.minversion = reader.getstring("minversion", None)
         config.provision_tox_env = name = reader.getstring("provision_tox_env", ".tox")
-        min_version = "tox >= {}".format(config.minversion or tox.__version__)
-        deps = self.ensure_requires_satisfied(config, requires_list, min_version)
+        min_version = "tox >= {}".format(config.minversion or Version(tox.__version__).public)
+        deps = self.ensure_requires_satisfied(config, config.requires, min_version)
         if config.run_provision:
             section_name = "testenv:{}".format(name)
             if section_name not in self._cfg.sections:
@@ -1291,8 +1338,8 @@ class ParseIni(object):
         # raise on unknown args
         self.config._parser.parse_cli(args=self.config.args, strict=True)
 
-    @staticmethod
-    def ensure_requires_satisfied(config, requires, min_version):
+    @classmethod
+    def ensure_requires_satisfied(cls, config, requires, min_version):
         missing_requirements = []
         failed_to_parse = False
         deps = []
@@ -1319,12 +1366,33 @@ class ParseIni(object):
                 missing_requirements.append(str(requirements.Requirement(require)))
         if failed_to_parse:
             raise tox.exception.BadRequirement()
+        if config.option.no_provision and missing_requirements:
+            msg = "provisioning explicitly disabled within {}, but missing {}"
+            if config.option.no_provision is not True:  # it's a path
+                msg += " and wrote to {}"
+                cls.write_requires_to_json_file(config)
+            raise tox.exception.Error(
+                msg.format(sys.executable, missing_requirements, config.option.no_provision)
+            )
         if WITHIN_PROVISION and missing_requirements:
             msg = "break infinite loop provisioning within {} missing {}"
             raise tox.exception.Error(msg.format(sys.executable, missing_requirements))
         config.run_provision = bool(len(missing_requirements))
         return deps
 
+    @staticmethod
+    def write_requires_to_json_file(config):
+        requires_dict = {
+            "minversion": config.minversion,
+            "requires": config.requires,
+        }
+        try:
+            with open(config.option.no_provision, "w", encoding="utf-8") as outfile:
+                json.dump(requires_dict, outfile, indent=4)
+        except TypeError:  # Python 2
+            with open(config.option.no_provision, "w") as outfile:
+                json.dump(requires_dict, outfile, indent=4, encoding="utf-8")
+
     def parse_build_isolation(self, config, reader):
         config.isolated_build = reader.getbool("isolated_build", False)
         config.isolated_build_env = reader.getstring("isolated_build_env", ".package")
@@ -1347,7 +1415,7 @@ class ParseIni(object):
         factors = set()
         if section in self._cfg:
             for _, value in self._cfg[section].items():
-                exprs = re.findall(r"^([\w{}\.!,-]+)\:\s+", value, re.M)
+                exprs = re.findall(r"^([\w{}.!,-]+):\s+", value, re.M)
                 factors.update(*mapcat(_split_factor_expr_all, exprs))
         return factors
 
@@ -1374,6 +1442,7 @@ class ParseIni(object):
                     "dict_setenv",
                     "argv",
                     "argvlist",
+                    "argv_install_command",
                 ):
                     meth = getattr(reader, "get{}".format(atype))
                     res = meth(env_attr.name, env_attr.default, replace=replace)
@@ -1397,9 +1466,10 @@ class ParseIni(object):
                 if env_attr.postprocess:
                     res = env_attr.postprocess(testenv_config=tc, value=res)
             except tox.exception.MissingSubstitution as e:
-                tc._missing_subs.append(e.name)
-                res = e.FLAG
-            setattr(tc, env_attr.name, res)
+                tc._missing_subs[env_attr.name] = res = e
+            # On Python 2, exceptions are handled in __getattr__
+            if not six.PY2 or not isinstance(res, Exception):
+                setattr(tc, env_attr.name, res)
             if atype in ("path", "string", "basepython"):
                 reader.addsubstitutions(**{env_attr.name: res})
         return tc
@@ -1446,6 +1516,11 @@ class ParseIni(object):
         if not env_list:
             env_list = all_envs
 
+        provision_tox_env = config.provision_tox_env
+        if config.provision_tox_env in env_list:
+            msg = "provision_tox_env {} cannot be part of envlist".format(provision_tox_env)
+            raise tox.exception.ConfigError(msg)
+
         package_env = config.isolated_build_env
         if config.isolated_build is True and package_env in all_envs:
             all_envs.remove(package_env)
@@ -1453,6 +1528,7 @@ class ParseIni(object):
         if config.isolated_build is True and package_env in env_list:
             msg = "isolated_build_env {} cannot be part of envlist".format(package_env)
             raise tox.exception.ConfigError(msg)
+
         return env_list, all_envs, _split_env(from_config), envlist_explicit
 
     @staticmethod
@@ -1463,7 +1539,7 @@ class ParseIni(object):
         The parser will see it as two different sections: [testenv:py36-cov], [testenv:py37-cov]
 
         """
-        factor_re = re.compile(r"\{\s*([\w\s,-]+)\s*\}")
+        factor_re = re.compile(r"{\s*([\w\s,-]+)\s*}")
         split_re = re.compile(r"\s*,\s*")
         to_remove = set()
         for section in list(config.sections):
@@ -1479,12 +1555,12 @@ class ParseIni(object):
 
 
 def _split_env(env):
-    """if handed a list, action="append" was used for -e """
+    """if handed a list, action="append" was used for -e"""
     if env is None:
         return []
     if not isinstance(env, list):
         env = [e.split("#", 1)[0].strip() for e in env.split("\n")]
-        env = ",".join([e for e in env if e])
+        env = ",".join(e for e in env if e)
         env = [env]
     return mapcat(_expand_envstr, env)
 
@@ -1558,7 +1634,15 @@ is_section_substitution = re.compile(r"{\[[^{}\s]+\]\S+?}").match
 
 
 class SectionReader:
-    def __init__(self, section_name, cfgparser, fallbacksections=None, factors=(), prefix=None):
+    def __init__(
+        self,
+        section_name,
+        cfgparser,
+        fallbacksections=None,
+        factors=(),
+        prefix=None,
+        posargs="",
+    ):
         if prefix is None:
             self.section_name = section_name
         else:
@@ -1569,6 +1653,7 @@ class SectionReader:
         self._subs = {}
         self._subststack = []
         self._setenv = None
+        self.posargs = posargs
 
     def get_environ_value(self, name):
         if self._setenv is None:
@@ -1656,11 +1741,27 @@ class SectionReader:
 
     def getargvlist(self, name, default="", replace=True):
         s = self.getstring(name, default, replace=False)
-        return _ArgvlistReader.getargvlist(self, s, replace=replace)
+        return _ArgvlistReader.getargvlist(self, s, replace=replace, name=name)
 
     def getargv(self, name, default="", replace=True):
         return self.getargvlist(name, default, replace=replace)[0]
 
+    def getargv_install_command(self, name, default="", replace=True):
+        s = self.getstring(name, default, replace=False)
+        if not s:
+            # This occurs when factors are used, and a testenv doesnt have
+            # a factorised value for install_command, most commonly occurring
+            # if setting platform is also used.
+            # An empty value causes error install_command must contain '{packages}'.
+            s = default
+
+        if "{packages}" in s:
+            s = s.replace("{packages}", r"\{packages\}")
+        if "{opts}" in s:
+            s = s.replace("{opts}", r"\{opts\}")
+
+        return _ArgvlistReader.getargvlist(self, s, replace=replace, name=name)[0]
+
     def getstring(self, name, default=None, replace=True, crossonly=False, no_fallback=False):
         x = None
         sections = [self.section_name] + ([] if no_fallback else self.fallbacksections)
@@ -1685,6 +1786,17 @@ class SectionReader:
         x = self._replace_if_needed(x, name, replace, crossonly)
         return x
 
+    def getposargs(self, default=None):
+        if self.posargs:
+            posargs = self.posargs
+            if sys.platform.startswith("win"):
+                posargs_string = list2cmdline([x for x in posargs if x])
+            else:
+                posargs_string = " ".join(shlex_quote(x) for x in posargs if x)
+            return posargs_string
+        else:
+            return default or ""
+
     def _replace_if_needed(self, x, name, replace, crossonly):
         if replace and x and hasattr(x, "replace"):
             x = self._replace(x, name=name, crossonly=crossonly)
@@ -1711,6 +1823,7 @@ class SectionReader:
             return value
 
         section_name = section_name if section_name else self.section_name
+        assert name
         self._subststack.append((section_name, name))
         try:
             replaced = Replacer(self, crossonly=crossonly).do_replace(value)
@@ -1731,7 +1844,7 @@ class Replacer:
         (?<!\\)[{]
         (?:(?P<sub_type>[^[:{}]+):)?    # optional sub_type for special rules
         (?P<substitution_value>(?:\[[^,{}]*\])?[^:,{}]*)  # substitution key
-        (?::(?P<default_value>[^{}]*))?   # default value
+        (?::(?P<default_value>([^{}]|\\{|\\})*))?   # default value
         [}]
         """,
         re.VERBOSE,
@@ -1757,6 +1870,10 @@ class Replacer:
 
         return expanded
 
+    @staticmethod
+    def _unescape(s):
+        return s.replace("\\{", "{").replace("\\}", "}")
+
     def _replace_match(self, match):
         g = match.groupdict()
         sub_value = g["substitution_value"]
@@ -1767,40 +1884,60 @@ class Replacer:
             start, end = match.span()
             return match.string[start:end]
 
-        # special case: all empty values means ":" which is os.pathsep
-        if not any(g.values()):
+        full_match = match.group(0)
+        # ":" is swallowed by the regex, so the raw matched string is checked
+        if full_match.startswith("{:"):
+            if full_match != "{:}":
+                raise tox.exception.ConfigError(
+                    "Malformed substitution with prefix ':': {}".format(full_match),
+                )
+
             return os.pathsep
 
+        default_value = g["default_value"]
         # special case: opts and packages. Leave {opts} and
         # {packages} intact, they are replaced manually in
         # _venv.VirtualEnv.run_install_command.
         if sub_value in ("opts", "packages"):
             return "{{{}}}".format(sub_value)
 
-        try:
-            sub_type = g["sub_type"]
-        except KeyError:
+        if sub_value == "posargs":
+            return self.reader.getposargs(default_value)
+
+        sub_type = g["sub_type"]
+        if sub_type == "posargs":
+            if default_value:
+                value = "{}:{}".format(sub_value, default_value)
+            else:
+                value = sub_value
+            return self.reader.getposargs(value)
+
+        if not sub_type and not sub_value:
             raise tox.exception.ConfigError(
-                "Malformed substitution; no substitution type provided",
+                "Malformed substitution; no substitution type provided. "
+                "If you were using `{}` for `os.pathsep`, please use `{:}`.",
             )
 
+        if not sub_type and not default_value and sub_value == "/":
+            return os.sep
+
         if sub_type == "env":
-            return self._replace_env(match)
+            return self._replace_env(sub_value, default_value)
         if sub_type == "tty":
             if is_interactive():
                 return match.group("substitution_value")
             return match.group("default_value")
+        if sub_type == "posargs":
+            return self.reader.getposargs(sub_value)
         if sub_type is not None:
             raise tox.exception.ConfigError(
                 "No support for the {} substitution type".format(sub_type),
             )
-        return self._replace_substitution(match)
+        return self._replace_substitution(sub_value)
 
-    def _replace_env(self, match):
-        key = match.group("substitution_value")
+    def _replace_env(self, key, default):
         if not key:
             raise tox.exception.ConfigError("env: requires an environment variable name")
-        default = match.group("default_value")
         value = self.reader.get_environ_value(key)
         if value is not None:
             return value
@@ -1815,7 +1952,7 @@ class Replacer:
             cfg = self.reader._cfg
             if section in cfg and item in cfg[section]:
                 if (section, item) in self.reader._subststack:
-                    raise ValueError(
+                    raise tox.exception.SubstitutionStackError(
                         "{} already in {}".format((section, item), self.reader._subststack),
                     )
                 x = str(cfg[section][item])
@@ -1828,8 +1965,7 @@ class Replacer:
 
         raise tox.exception.ConfigError("substitution key {!r} not found".format(key))
 
-    def _replace_substitution(self, match):
-        sub_key = match.group("substitution_value")
+    def _replace_substitution(self, sub_key):
         val = self.reader._subs.get(sub_key, None)
         if val is None:
             val = self._substitute_from_other_section(sub_key)
@@ -1844,7 +1980,7 @@ def is_interactive():
 
 class _ArgvlistReader:
     @classmethod
-    def getargvlist(cls, reader, value, replace=True):
+    def getargvlist(cls, reader, value, replace=True, name=None):
         """Parse ``commands`` argvlist multiline string.
 
         :param SectionReader reader: reader to be used.
@@ -1866,10 +2002,10 @@ class _ArgvlistReader:
             current_command += line
 
             if is_section_substitution(current_command):
-                replaced = reader._replace(current_command, crossonly=True)
-                commands.extend(cls.getargvlist(reader, replaced))
+                replaced = reader._replace(current_command, crossonly=True, name=name)
+                commands.extend(cls.getargvlist(reader, replaced, name=name))
             else:
-                commands.append(cls.processcommand(reader, current_command, replace))
+                commands.append(cls.processcommand(reader, current_command, replace, name=name))
             current_command = ""
         else:
             if current_command:
@@ -1882,13 +2018,7 @@ class _ArgvlistReader:
         return commands
 
     @classmethod
-    def processcommand(cls, reader, command, replace=True):
-        posargs = getattr(reader, "posargs", "")
-        if sys.platform.startswith("win"):
-            posargs_string = list2cmdline([x for x in posargs if x])
-        else:
-            posargs_string = " ".join([shlex_quote(x) for x in posargs if x])
-
+    def processcommand(cls, reader, command, replace=True, name=None):
         # Iterate through each word of the command substituting as
         # appropriate to construct the new command string. This
         # string is then broken up into exec argv components using
@@ -1896,19 +2026,14 @@ class _ArgvlistReader:
         if replace:
             newcommand = ""
             for word in CommandParser(command).words():
-                if word == "{posargs}" or word == "[]":
-                    newcommand += posargs_string
+                if word == "[]":
+                    newcommand += reader.getposargs()
                     continue
-                elif word.startswith("{posargs:") and word.endswith("}"):
-                    if posargs:
-                        newcommand += posargs_string
-                        continue
-                    else:
-                        word = word[9:-1]
+
                 new_arg = ""
-                new_word = reader._replace(word)
-                new_word = reader._replace(new_word)
-                new_word = new_word.replace("\\{", "{").replace("\\}", "}")
+                new_word = reader._replace(word, name=name)
+                new_word = reader._replace(new_word, name=name)
+                new_word = Replacer._unescape(new_word)
                 new_arg += new_word
                 newcommand += new_arg
         else:
